@@ -1,12 +1,15 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from trytond.model import Model, ModelView, ModelSQL, fields
+from trytond.model import fields
 from trytond.pyson import Eval
-from trytond.pool import Pool
+from trytond.pool import Pool, PoolMeta
+
+__all__ = ['FiscalYear', 'Period', 'Move', 'Reconciliation']
+__metaclass__ = PoolMeta
 
 
-class FiscalYear(ModelSQL, ModelView):
-    _name = 'account.fiscalyear'
+class FiscalYear:
+    __name__ = 'account.fiscalyear'
     out_invoice_sequence = fields.Many2One('ir.sequence.strict',
         'Customer Invoice Sequence', required=True,
         domain=[
@@ -62,12 +65,13 @@ class FiscalYear(ModelSQL, ModelView):
             'company': Eval('company'),
             }, depends=['company'])
 
-    def __init__(self):
-        super(FiscalYear, self).__init__()
-        self._constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(FiscalYear, cls).__setup__()
+        cls._constraints += [
             ('check_invoice_sequences', 'different_invoice_sequence'),
-        ]
-        self._error_messages.update({
+            ]
+        cls._error_messages.update({
             'change_invoice_sequence': 'You can not change ' \
                     'the invoice sequence if there is already ' \
                     'an invoice opened in the fiscalyear',
@@ -75,44 +79,41 @@ class FiscalYear(ModelSQL, ModelView):
                     'invoice sequences per fiscal year!',
             })
 
-    def check_invoice_sequences(self, ids):
-        for fiscalyear in self.browse(ids):
-            for sequence in ('out_invoice_sequence', 'in_invoice_sequence',
-                    'out_credit_note_sequence', 'in_credit_note_sequence'):
-                if self.search([
-                    (sequence, '=', fiscalyear[sequence].id),
-                    ('id', '!=', fiscalyear.id),
-                    ]):
-                    return False
+    def check_invoice_sequences(self):
+        for sequence in ('out_invoice_sequence', 'in_invoice_sequence',
+                'out_credit_note_sequence', 'in_credit_note_sequence'):
+            if self.search([
+                        (sequence, '=', getattr(self, sequence).id),
+                        ('id', '!=', self.id),
+                        ]):
+                return False
         return True
 
-    def write(self, ids, vals):
-        invoice_obj = Pool().get('account.invoice')
-
-        if isinstance(ids, (int, long)):
-            ids = [ids]
+    @classmethod
+    def write(cls, fiscalyears, vals):
+        Invoice = Pool().get('account.invoice')
 
         for sequence in ('out_invoice_sequence', 'in_invoice_sequence',
                 'out_credit_note_sequence', 'in_credit_note_sequence'):
             if vals.get(sequence):
-                for fiscalyear in self.browse(ids):
-                    if fiscalyear[sequence] and \
-                            fiscalyear[sequence].id != \
-                            vals[sequence]:
-                        if invoice_obj.search([
-                            ('invoice_date', '>=', fiscalyear.start_date),
-                            ('invoice_date', '<=', fiscalyear.end_date),
-                            ('number', '!=', None),
-                            ('type', '=', sequence[:-9]),
-                            ]):
-                            self.raise_user_error('change_invoice_sequence')
-        return super(FiscalYear, self).write(ids, vals)
-
-FiscalYear()
+                for fiscalyear in fiscalyears:
+                    if (getattr(fiscalyear, sequence)
+                            and (getattr(fiscalyear, sequence).id !=
+                                vals[sequence])):
+                        if Invoice.search([
+                                    ('invoice_date', '>=',
+                                        fiscalyear.start_date),
+                                    ('invoice_date', '<=',
+                                        fiscalyear.end_date),
+                                    ('number', '!=', None),
+                                    ('type', '=', sequence[:-9]),
+                                    ]):
+                            cls.raise_user_error('change_invoice_sequence')
+        super(FiscalYear, cls).write(fiscalyears, vals)
 
 
-class Period(ModelSQL, ModelView):
-    _name = 'account.period'
+class Period:
+    __name__ = 'account.period'
     out_invoice_sequence = fields.Many2One('ir.sequence.strict',
         'Customer Invoice Sequence',
         domain=[('code', '=', 'account.invoice')],
@@ -150,12 +151,13 @@ class Period(ModelSQL, ModelView):
             },
         depends=['type'])
 
-    def __init__(self):
-        super(Period, self).__init__()
-        self._constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(Period, cls).__setup__()
+        cls._constraints += [
             ('check_invoice_sequences', 'check_invoice_sequences'),
-        ]
-        self._error_messages.update({
+            ]
+        cls._error_messages.update({
             'change_invoice_sequence': 'You can not change ' \
                     'the invoice sequence if there is already ' \
                     'an invoice opened in the period',
@@ -164,86 +166,82 @@ class Period(ModelSQL, ModelView):
                     'in the same company!',
             })
 
-    def check_invoice_sequences(self, ids):
-        for period in self.browse(ids):
-            for sequence in ('out_invoice_sequence', 'in_invoice_sequence',
-                    'out_credit_note_sequence', 'in_credit_note_sequence'):
-                if self.search([
-                    (sequence, '=', period[sequence].id),
-                    ('fiscalyear', '!=', period.fiscalyear.id),
-                    ]):
-                    return False
-                if period[sequence].company \
-                        and period[sequence].company.id != \
-                        period.fiscalyear.company.id:
-                    return False
+    def check_invoice_sequences(self):
+        for sequence_name in ('out_invoice_sequence', 'in_invoice_sequence',
+                'out_credit_note_sequence', 'in_credit_note_sequence'):
+            sequence = getattr(self, sequence_name)
+            if self.search([
+                        (sequence_name, '=', sequence.id),
+                        ('fiscalyear', '!=', self.fiscalyear.id),
+                        ]):
+                return False
+            if (sequence.company
+                    and sequence.company != self.fiscalyear.company):
+                return False
         return True
 
-    def create(self, vals):
-        fiscalyear_obj = Pool().get('account.fiscalyear')
+    @classmethod
+    def create(cls, vals):
+        FiscalYear = Pool().get('account.fiscalyear')
         vals = vals.copy()
         if vals.get('fiscalyear'):
-            fiscalyear = fiscalyear_obj.browse(vals['fiscalyear'])
+            fiscalyear = FiscalYear(vals['fiscalyear'])
             for sequence in ('out_invoice_sequence', 'in_invoice_sequence',
                     'out_credit_note_sequence', 'in_credit_note_sequence'):
                 if not vals.get(sequence):
-                    vals[sequence] = fiscalyear[sequence].id
-        return super(Period, self).create(vals)
+                    vals[sequence] = getattr(fiscalyear, sequence).id
+        return super(Period, cls).create(vals)
 
-    def write(self, ids, vals):
-        invoice_obj = Pool().get('account.invoice')
+    @classmethod
+    def write(cls, periods, vals):
+        Invoice = Pool().get('account.invoice')
 
-        if isinstance(ids, (int, long)):
-            ids = [ids]
-
-        for sequence in ('out_invoice_sequence', 'in_invoice_sequence',
+        for sequence_name in ('out_invoice_sequence', 'in_invoice_sequence',
                 'out_credit_note_sequence', 'in_credit_note_sequence'):
-            if vals.get(sequence):
-                for period in self.browse(ids):
-                    if period[sequence] and \
-                            period[sequence].id != \
-                            vals[sequence]:
-                        if invoice_obj.search([
-                            ('invoice_date', '>=', period.start_date),
-                            ('invoice_date', '<=', period.end_date),
-                            ('number', '!=', None),
-                            ('type', '=', sequence[:-9]),
-                            ]):
-                            self.raise_user_error('change_invoice_sequence')
-        return super(Period, self).write(ids, vals)
-
-Period()
+            if vals.get(sequence_name):
+                for period in periods:
+                    sequence = getattr(period, sequence_name)
+                    if (sequence and
+                            sequence.id != vals[sequence]):
+                        if Invoice.search([
+                                    ('invoice_date', '>=', period.start_date),
+                                    ('invoice_date', '<=', period.end_date),
+                                    ('number', '!=', None),
+                                    ('type', '=', sequence_name[:-9]),
+                                    ]):
+                            cls.raise_user_error('change_invoice_sequence')
+        super(Period, cls).write(periods, vals)
 
 
-class Reconciliation(Model):
-    _name = 'account.move.reconciliation'
+class Move:
+    __name__ = 'account.move'
 
-    def create(self, values):
-        pool = Pool()
-        invoice_obj = pool.get('account.invoice')
+    @classmethod
+    def _get_origin(cls):
+        return super(Move, cls)._get_origin() + ['account.invoice']
 
-        result = super(Reconciliation, self).create(values)
-        reconciliation = self.browse(result)
+
+class Reconciliation:
+    __name__ = 'account.move.reconciliation'
+
+    @classmethod
+    def create(cls, values):
+        Invoice = Pool().get('account.invoice')
+        reconciliation = super(Reconciliation, cls).create(values)
         move_ids = set(l.move.id for l in reconciliation.lines)
-        invoice_ids = invoice_obj.search([
+        invoices = Invoice.search([
                 ('move', 'in', list(move_ids)),
                 ])
-        invoice_obj.process(invoice_ids)
-        return result
+        Invoice.process(invoices)
+        return reconciliation
 
-    def delete(self, ids):
-        pool = Pool()
-        invoice_obj = pool.get('account.invoice')
+    @classmethod
+    def delete(cls, reconciliations):
+        Invoice = Pool().get('account.invoice')
 
-        if isinstance(ids, (int, long)):
-            ids = [ids]
-        reconciliations = self.browse(ids)
         move_ids = set(l.move.id for r in reconciliations for l in r.lines)
-        invoice_ids = invoice_obj.search([
+        invoices = Invoice.search([
                 ('move', 'in', list(move_ids)),
                 ])
-        result = super(Reconciliation, self).delete(ids)
-        invoice_obj.process(invoice_ids)
-        return result
-
-Reconciliation()
+        super(Reconciliation, cls).delete(reconciliations)
+        Invoice.process(invoices)
